#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Dec 12 13:47:48 2017

@author: lessig
"""

import numpy as np

from ray import Ray
from camera import Camera
from intersection import Intersection
from shading import shade


from multiprocessing import Pool

def renderTileHelper( args):
    return renderTile( *args)
    

def renderTile( scene, cam, res_x, res_y, taskid, numtasks) :  

    tilesize = np.int( np.floor(res_x / numtasks))
    xstart = taskid * tilesize
    xend = (taskid+1)* tilesize

    for ix in range( xstart, xend) :
        for iy in range( 0, res_y) :
    
            r = cam.generateRay( ix, iy)
    
            # trace ray through scene
            isect = Intersection() 
            for obj in scene :
                obj.intersect( r, isect)
    
            if( r.t < 10000.) :
               col = shade( isect.p, isect.n, isect.rho, scene, -r.d, 4)
               cam.image[ix,iy] = col

    return cam.image
    

def render( res_x, res_y, scene, num_threads) :

    cam = Camera( 30, res_x, res_y)
    
    args = []
    for i in range( num_threads) :
        args.append( [scene, cam, res_x, res_y, i, num_threads])
        
    p = Pool( num_threads)
    ress = p.map( renderTileHelper, args)
    
    im = ress[0]
    for i in range( 1, num_threads) :
        im += ress[i]
    
    return im
